/* Name: main.c firmware
 * Project: Basic USB thermometer based on hid-data example from avrusb project (Author: Christian Starkjohann)
 * Author: Russel McAuliffe
 * Creation Date: 2008-06-11
 * Copyright: (c) 2008 by sentientresearch ltd
 * License: GNU GPL v2 (see License.txt) or proprietary (CommercialLicense.txt)
 */


#include <util/delay.h>     /* for _delay_ms() */
#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <avr/eeprom.h>
#include <avr/wdt.h>

#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
#include "oddebug.h"        /* This is also an example for using debug macros */

unsigned char lval;
unsigned char hval;
unsigned char lval2;
unsigned char hval2;
unsigned char lval3;
unsigned char hval3;
unsigned char dbg;


/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[22] = {    /* USB report descriptor */
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x80,                    //   REPORT_COUNT (128)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};
/* Since we define only one feature report, we don't use report-IDs (which
 * would be the first byte of the report). The entire report consists of 128
 * opaque data bytes.
 */

/* The following variables store the status of the current data transfer */
static uchar    currentAddress;
static uchar    bytesRemaining;

/* ------------------------------------------------------------------------- */



int ad_ConversionComplete (void) {
	return (!(ADCSRA & (1 << ADSC)));
}

 int ad_Read10bit (void) {
 	
// 	lval=ADCL;
// 	hval=ADCH;
 	
// 	return (lval | hval << 8);
 }
 
 int ad_Read8bit (void) {
 	return ad_Read10bit() >> 2;
 }
 
// void ad_SetChannel (uchar mux) {
// 	ADMUX = (ADMUX & ~ADC_MUX_MASK) | (mux & ADC_MUX_MASK);		// set channel
// }
 
 void ad_StartConversion () {
 			ADCSRA |= (1 << ADIF);			// clear hardware "conversion complete" flag 
			ADCSRA |= (1 << ADSC);			// start conversion
}


void init_ADC(void)
	{





	}






void read_ADC(int line)
	{
	unsigned char value;

	
	ADCSRA=0x8F;         //Enable ADC, ADC interrupt, prescaler 128


	ADMUX = line;			// start by selecting channel 0
	//ADMUX = ADMUX | 0x40;		// select the voltage reference
    	ADMUX = ADMUX | 0xC0;		// select the voltage reference
//	ADMUX = ADMUX | 0x20;		// set for Left Justified
	
	ADCSR = 0x07;			// select the ADC clock frequency

	ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); 	
	ADCSR = ADCSR | 0x80;		// enable the ADC
	
	ADCSR = ADCSR | 0x40;			// start a conversion	
	while( (ADCSR & 0x10) == 0 );		// wait for conversion to be completed
//	lval = ADCL;
//	hval = ADCH;				// get the upper 8-bits
	ADCSR = ADCSR | 0x10;			// clear the flag

	ADCSR = ADCSR | 0x40;			// start a conversion	
	while( (ADCSR & 0x10) == 0 );		// wait for conversion to be completed
	lval = ADCL;
	hval = ADCH;				// get the upper 8-bits
	ADCSR = ADCSR | 0x10;			// clear the flag
	
	return value;
	}


 void OLDread_ADC(int line)
	{
  
      	
	ADCSR = ADCSR | 0x40;			// start a conversion
	
	while( (ADCSR & 0x10) == 0 );		// wait for conversion to be completed
	
	lval = ADCL;
	hval = ADCH;				// get the upper 8-bits
	ADCSR = ADCSR | 0x10;			// clear the flag
	//return hvalue;
	//return (hvalue * 256)+lvalue; 
	}



/* usbFunctionRead() is called when the host requests a chunk of data from
 * the device. For more information see the documentation in usbdrv/usbdrv.h.
 */
uchar   usbFunctionRead(uchar *data, uchar len)
{

    if(len > bytesRemaining)
        len = bytesRemaining;
    eeprom_read_block(data, (uchar *)0 + currentAddress, len);
    currentAddress += len;
    bytesRemaining -= len;
    

    if(bytesRemaining==0x78) {  // only measure for the first byte sent
      read_ADC(2);    
      data[0]=hval;
      data[1]=lval;

      read_ADC(3);    
      data[2]=hval;
      data[3]=lval;

      read_ADC(4);    
      data[4]=hval;
      data[5]=lval;


      read_ADC(5);    
      data[6]=hval;
      data[7]=lval;


    } else {
      hval=1;
      lval=1;
      
    }
    

        

    return len;
}

/* usbFunctionWrite() is called when the host sends a chunk of data to the
 * device. For more information see the documentation in usbdrv/usbdrv.h.
 */
uchar   usbFunctionWrite(uchar *data, uchar len)
{
    if(bytesRemaining == 0)
        return 1;               /* end of transfer */
    if(len > bytesRemaining)
        len = bytesRemaining;
    eeprom_write_block(data, (uchar *)0 + currentAddress, len);
    currentAddress += len;
    bytesRemaining -= len;
    return bytesRemaining == 0; /* return 1 if this was the last chunk */
}

/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* HID class request */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            /* since we have only one report type, we can ignore the report-ID */
            bytesRemaining = 128;
            currentAddress = 0;
            return USB_NO_MSG;  /* use usbFunctionRead() to obtain data */
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            /* since we have only one report type, we can ignore the report-ID */
            bytesRemaining = 128;
            currentAddress = 0;
            return USB_NO_MSG;  /* use usbFunctionWrite() to receive data from host */
        }
    }else{
        /* ignore vendor type requests, we don't use any */
    }
    return 0;
}

/* ------------------------------------------------------------------------- */


int main(void)
{
uchar   i;

    wdt_enable(WDTO_1S);
    /* Even if you don't use the watchdog, turn it off here. On newer devices,
     * the status of the watchdog (on/off, period) is PRESERVED OVER RESET!
     */

    DBG1(0x00, 0, 0);       /* debug output: main starts */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */

    DDRC  	= 0x00;		// set all pins to input
//    PORTC	= 0x0ff;		// make sure pull-up resistors are turned on
    PORTC	= 0x00;		// make sure pull-up resistors are turned off


    odDebugInit();
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }
    usbDeviceConnect();
    sei();
    DBG1(0x01, 0, 0);       /* debug output: main loop starts */




    for(;;){                /* main event loop  - will run through for each block of 8 bytes of usb packet*/
                
        DBG1(0x02, 0, 0);   /* debug output: main loop iterates */
        wdt_reset();
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
